/*
    First Course in Java, EDP 135277
    Homework #6 (option 3)
    Tim Ong

   CopsAndRobbers
   Example of a possible "cops and robbers" game, in which all the players
   are ultimately 'characters.' Some of the characters carry guns, some are
   good guys, some are bad guys, some are simply bystanders, and some may even
   have superpowers.

   Demonstrates
   - multiple inheritance through the use of interfaces.

    Sample Output:
   CopsAndRobbers
   ............................................
   CREATE

   Cop Kojak has been created.
   -- health: 51
   -- accuracy: 43
   Robber Scarface has been created.
   -- health: 60
   -- accuracy: 24
   Bystander Joe has been created.
   -- health: 88
   Superhero Superman has been created.
   -- health: 60
   -- superpower: x-ray vision
   ............................................
   ACTION

   Scarface is stealing something.
   Joe is getting in the way.
   Scarface fired a shot at Joe
   Missed
   Kojak fired a shot at Scarface
   *HIT*
   Scarface has been hit...52 health left.
   Superman uses his x-ray vision superpower to save the day!
   Officer Kojak is keeping the peace.

   END
*/

// all good guys can do good.
interface GoodGuy
{
   void doGood();
}

// all bad guys can do bad.
interface BadGuy
{
   void doBad();
}

// a character that carries a gun can shoot at a target
interface CarriesGun
{
   void shoot(Character target);
}

/*
   Character
   All characters have a name, have health, and can be wounded.
*/
class Character
{
   protected String name;
   protected int health;

   // constructor
   public Character(String name)
   {
      this.name = name;
      this.health = (int)(Math.random() * 50) + 50; //
   }

   public int getHealth()
   {
      return this.health;
   }

   protected void wounded(int amount)
   {
      this.health -= amount;

      if (this.health > 0)
      {
         System.out.println(this.name+" has been hit..."+health+" health left.");
      }
      else
      {
         System.out.println(this.name+" has been killed.");
      }
   }
}

/*
   Gunslinger
   Special characters that carry guns.
*/
class Gunslinger extends Character implements CarriesGun
{
   protected int accuracy;
   private int hit;

   public Gunslinger(String name)
   {
      super(name);
      this.accuracy = (int)(Math.random () * 50) + 1;
   }

   public void shoot(Character target)
   {
      // random number (1-100)
      hit = (int)(Math.random () *100) + 1;

      System.out.println(name+" fired a shot at "+target.name);

      if (hit > accuracy)
      {
         System.out.println ("*HIT*");

         target.wounded((int)(Math.random () * 20));
      } else
      {
         System.out.println("Missed");
      }
   }
}

/*
   Cop
   A gunslinger that is also a good guy.
*/
class Cop extends Gunslinger implements GoodGuy
{
   public Cop (String name)
   {
      super(name);

      System.out.println("Cop "+name + " has been created.");
      System.out.println("-- health: "+health);
      System.out.println("-- accuracy: "+accuracy);
   }

   public void doGood()
   {
      System.out.println ("Officer "+name+" is keeping the peace.");
   }
}

/*
   Robber
   A gunslinger that is also a bad guy.
*/
class Robber extends Gunslinger implements BadGuy
{
   public Robber(String name)
   {
      super(name);

      System.out.println("Robber "+name + " has been created.");
        System.out.println("-- health: "+health);
        System.out.println("-- accuracy: "+accuracy);
   }

   public void doBad()
   {
      System.out.println(name + " is stealing something.");
   }
}

/*
   Superhero
   A character that is a good guy, and that also has a superpower.
*/
class Superhero extends Character implements GoodGuy
{
   public String superpower;

   public Superhero(String name, String superpower)
   {
      super(name);

      this.superpower = superpower;

      System.out.println("Superhero "+name + " has been created.");
      System.out.println("-- health: "+health);
      System.out.println("-- superpower: "+superpower);
   }

   public void doGood()
   {
      System.out.println(name + " uses his "+superpower+" superpower to save the day!");
   }

}

/*
   Bystander
   A character that gets in the way.
*/
class Bystander extends Character
{
   public Bystander(String name)
   {
      super(name);

      System.out.println("Bystander "+name + " has been created.");
        System.out.println("-- health: "+health);
   }

   public void getInTheWay()
   {
      System.out.println(name+" is getting in the way.");
   }

}

/*
   CopsAndRobbers
   Creates a Cop, a Robber, a Bystander, and a Superhero, and demonstrates how
   they interact together (like in a game, perhaps).
*/
class CopsAndRobbers
{
   public static void main(String[] args)
   {

      System.out.println("CopsAndRobbers");
      System.out.println("............................................");
      System.out.println("CREATE");
        System.out.println();

      // instantiate the players
      Cop myCop = new Cop("Kojak");
      Robber myRobber = new Robber("Scarface");
      Bystander myBystander = new Bystander("Joe");
      Superhero mySuperhero = new Superhero("Superman", "x-ray vision");

      System.out.println("............................................");
      System.out.println("ACTION");
      System.out.println();

      // now each player does something
      myRobber.doBad();
      myBystander.getInTheWay();
      myRobber.shoot(myBystander);
      myCop.shoot(myRobber);
      mySuperhero.doGood();
      myCop.doGood();

      System.out.println();
      System.out.println("END");

   }
}