/*
   First Course in Java, EDP 135277
   Homework #5
   Tim Ong

   EmployeeDemo
   Prompts user for an employee name, then asks whether they are an
   employee or a contractor. If employee, prompt for title and create
   a new Employee. If contractor, prompt for project and create a
   new Contractor.

   Demonstrates:
   - inheritance (Employee and Contractor are siblings which extend Person)
   - use of access specifiers
   - use of "getters" and "setters"

   Sample Output:
      Enter some information to create a new Employee OR a new Contractor.
      Last Name: Smith
      First Name: John
      (e)mployee or (c)ontractor? c
      Creating a contractor.
      Project: Site Redesign

      Contractor John Smith has been created for the Site Redesign project

      Press 'q' to quit or any other key to continue...
*/
import java.io.*;

/*
   InputField - wraps BufferedReader
*/
class InputField
{
   String input = "";
   String str;
   BufferedReader br = new BufferedReader (new InputStreamReader (System.in));

   // prompt the user for some input and return the result
   public String prompt (String text)
   {

      do
      {
         System.out.print (text+" ");
         input = getInput ();
      } while (input.equals(""));

      return input;
   }

   // get input
   public String getInput ()
   {
      try
      {
         str = br.readLine();
      }
      catch (IOException e)
      {
         str =  "I/O Exception " + e;
      }

      return str;
   }
}

/*
   Person
   Information about a person
*/
class Person
{
   private  String lastName;
   private String firstName;

   // constructor with formal parameters
   Person (String last, String first)
   {

      this.setLastName (last);
      this.setFirstName (first);
   }

   // return the entire name
   protected String getFullName ()
   {
      return this.firstName + " " + this.lastName;
   }

   // return the first name only
   protected String getFirstName ()
   {
      return this.firstName;
   }

   // return the last name only
   protected String getLastName ()
   {
      return this.lastName;
   }

   // set the last name
   private void setLastName (String last)
   {
      this.lastName = last;
   }

   // set the first name
   private void setFirstName (String first)
   {
      this.firstName = first;
   }

}

/*
   Employee
   Extends Person
*/
class Employee extends Person
{
   private String title;

   Employee (String last, String first, String title)
   {
      super (last, first);
      this.setTitle (title);
   }

   // return the title
   public String getTitle ()
   {
      return this.title;
   }

   // set the title
   private void setTitle (String title)
   {
      this.title = title;
   }


}

/*
   Contractor
   Extends Person
*/
class Contractor extends Person
{
   private String project;

   Contractor (String last, String first, String project)
   {
      super (last, first);
      this.setProject (project);
   }

   // return the current project
   public String getProject ()
   {
      return this.project;
   }

   // set the project
   private void setProject (String project)
   {
      this.project = project;
   }

}

/*
   EmployeeDemo
   Input some information to create a new employee
*/
class EmployeeDemo
{
   public static void main (String[] args)
   {
      String str, lastName, firstName, employeeType, title, project;

      Employee myEmployee;
      Contractor myContractor;

      // create an InputField
      InputField myInput = new InputField ();

      System.out.println ("Enter some information to create a new Employee OR a new Contractor.");

      do {

         // prompt for the field names
         lastName = myInput.prompt("Last Name:");
         firstName = myInput.prompt ("First Name:");

         do {
            // prompt for employee type
            employeeType = myInput.prompt ("(e)mployee or (c)ontractor?");
         } while (!(employeeType.equals("e") || employeeType.equals("c")));

         if (employeeType.equals("e")) {
            System.out.println ("Creating an employee.");
            title = myInput.prompt ("Title:");
            myEmployee = new Employee (lastName, firstName, title);

            //-------------------------------------------
            // ILLEGAL - can't access private property
            //-------------------------------------------
            //System.out.println ("***"+myEmployee.title);

            System.out.println();
            System.out.println (myEmployee.getTitle()+" "+myEmployee.getFullName()+" has been created.");
         } else if (employeeType.equals("c")) {
            System.out.println ("Creating a contractor.");
            project = myInput.prompt ("Project:");
            myContractor = new Contractor (lastName, firstName, project);
            System.out.println();
            System.out.println ("Contractor "+myContractor.getFullName()+" has been created for the "+myContractor.getProject()+" project");
         }

         System.out.println ();
         System.out.print("Press 'q' to quit or any other key to continue...");
         str = myInput.getInput();
         System.out.println ();

      } while (!(str.equals("q") || str.equals("Q")));

      System.out.println ("Goodbye.");
   }
}
